#include <string.h>
#include "arg.h"
#include "log.h"
#include "dep.h"
#include "generator.h"

/* for strue/sfalse */
#include "db.h"

int notree = 0;
char *notree_reason = NULL;

/* Runs when a custom command line argument is found
 returns true if no furhter argument processing should be done */
int hook_custom_arg(const char *key, const char *value)
{
	if (strcmp(key, "without-c99tree") == 0) {
		notree = 2;
		notree_reason = "it was disabled from the command line (--without-c99tree)";
	}
	return 0;
}


/* Runs before anything else */
int hook_preinit()
{
	return 0;
}

/* Runs after initialization */
int hook_postinit()
{
	report("Configuring libporty.\n");
	logprintf(0, "Configuring libporty.\n");
	return 0;
}

/* Runs after all arguments are read and parsed */
int hook_postarg()
{
	return 0;
}

/* Runs when things should be detected for the host system */
int hook_detect_host()
{
	require("fstools/ar", 1, 1);
	require("fstools/ranlib", 1, 1);
	require("fstools/mkdir", 1, 1);
	return 0;
}

/* Runs when things should be detected for the target system */
int hook_detect_target()
{
	if (require("libs/parser/expat/ldflags", 1, 0)) {
		notree = 1;
		notree_reason = "of missing libexpat";
	}

	if (!notree && require("parsgen/flex", 1, 0)) {
		notree = 1;
		notree_reason = "of missing flex";
	}

	if (!notree && require("parsgen/bison", 1, 0)) {
		notree = 1;
		notree_reason = "of missing bison";
	}

	require("sys/byte_order", 1, 1);
	require("cc/inline", 1, 0);

/* for bgtask */
	require("libs/lpthread", 1, 0);

	require("libs/time/usleep/presents", 1, 0);
	require("libs/time/Sleep/presents", 1, 0);

	/* find a suitable way to query subsecond time - prefer gettimeofday over ftime */
	if (require("libs/time/gettimeofday/presents", 1, 0))
		require("libs/time/ftime/presents", 1, 1);

	/* for P_quick_exit() */
	if (require("cc/_exit/presents", 1, 0)) {
		require("signal/raise/presents", 1, 1);
		require("signal/names/*", 1, 1);
	}


	/* socket library for net */
	require("libs/socket/poll/presents", 1, 0);
	require("libs/socket/select/presents", 1, 0);
	require("libs/socket/closesocket/presents", 1, 0);
	require("libs/vdprintf", 1, 0);
	require("libs/vsnprintf", 1, 0);
	require("libs/snprintf", 1, 0);

	require("libs/socket/socketpair/presents", 1, 0);
	require("libs/socket/ioctl/presents", 1, 0);
	require("libs/socket/ioctl/fionbio/presents", 1, 1); /* fatal because there is no alternative at the moment */
	require("libs/socket/lac/presents", 1, 1);
	require("libs/socket/recvsend/presents", 1, 1);
	require("libs/socket/readwrite/presents", 1, 0);
	require("libs/socket/shutdown/presents", 1, 1);
	require("libs/socket/ntoh/presents", 1, 1);
	if (require("libs/socket/getaddrinfo/presents", 1, 0))
		put("local/net/use_porty_resolver", strue);
	else
		put("local/net/use_porty_resolver", sfalse);


	/* Try to find constants for shutdown - SHUT_* on POSIX and SD_* on win32.
	   If none found we will fall back using hardwired integers. */
	if (require("libs/socket/SHUT/presents", 1, 0))
		require("libs/socket/SD/presents", 1, 0);

	if (require("libs/fs/readdir/presents", 1, 0))
		require("libs/fs/findnextfile/presents", 1, 1);


	/* detect integer types for host/types.h */
		require("sys/types/size/1_u_int", 1, 1);
		require("sys/types/size/2_u_int", 1, 1);
		require("sys/types/size/4_u_int", 1, 1);
		require("sys/types/size/1_s_int", 1, 1);
		require("sys/types/size/2_s_int", 1, 1);
		require("sys/types/size/4_s_int", 1, 1);

		require("sys/ptrwidth", 1, 1);
		require("sys/types/size_t/broken", 1, 0);
		require("sys/types/off_t/broken", 1, 0);
		require("sys/types/ptrdiff_t/broken", 1, 0);


	/* detect math lib corner cases for math_wrap.c */
	require("libs/math/cc/log/*", 1, 0);

	if (notree)
		put("local/c99tree/enabled", sfalse);
	else
		put("local/c99tree/enabled", strue);

	return 0;
}

/* Runs after detection hooks, should generate the output (Makefiles, etc.) */
int hook_generate()
{
	printf("Generating Makefiles for src/...\n");
	generate("../src/Makefile.in", "../src/Makefile");
	printf("Generating Makefiles for c99tree...\n");
	generate("../src/porty/c99tree/Makefile.config.in", "../src/porty/c99tree/Makefile.config");
	printf("Generating config.h for libporty...\n");
	generate("../src/libporty/config.h.in", "../src/libporty/config.h");
	generate("../src/libporty/Makefile.config.in", "../src/libporty/Makefile.config");

	generate("../src/libporty/net/os_includes.h.in", "../src/libporty/net/os_includes.h");
	generate("../src/libporty/net_resolver/resolver_inc.h.in", "../src/libporty/net_resolver/resolver_inc.h");
	generate("../src/libporty/host/fs_includes.h.in", "../src/libporty/host/fs_includes.h");
	generate("../src/libporty/host/time_includes.h.in", "../src/libporty/host/time_includes.h");
	generate("../src/libporty/host/types.h.in", "../src/libporty/host/types.h");
	generate("../src/libporty/net_async/Makefile.in", "../src/libporty/net_async/Makefile");
	generate("../src/regression/libporty/Makefile.in", "../src/regression/libporty/Makefile");


	printf("\n\n=== Result ===\n");
	if (notree) {
		printf("libporty is configured.\n");
		printf("Can't configure (and build) c99tree because %s\n", notree_reason);
		if (notree == 1)
			printf("In case c99tree and porty is needed, please install the missing software and rerun configure\n");
	}
	else
		printf("libporty and c99tree and porty are configured.\n");
	return 0;
}

/* Runs before everything is uninitialized */
void hook_preuninit()
{
}

/* Runs at the very end, when everything is already uninitialized */
void hook_postuninit()
{
}

