/*
    libporty - /etc/hosts file parser
    Copyright (C) 2012  Tibor 'Igor2' Palinkas

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Project page: http://repo.hu/projects/libporty
    Author: libporty@igor2.repo.hu
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <libporty/net/uninit_chain.h>
#include "net_mtime.h"


static char *hosts_fn = "/etc/hosts";
static P_time_t hosts_seen = 0;

typedef struct host_s host_t;
struct host_s {
	char *ip;
	char *name;
	host_t *next;
};
static host_t *hosts = NULL;

static char *strclone(const char *i)
{
	char *o;
	int l;

	l = strlen(i);
	o = malloc(l+1);
	memcpy(o, i, l+1);
	return o;
}

static void put(const char *ip, const char *name)
{
	host_t *h;

	h = malloc(sizeof(host_t));
	h->ip = strclone(ip);
	h->name = strclone(name);
	h->next = hosts;
	hosts = h;
}

/* whether uninit chain is already set up */
static in_chain = 0;

static void free_all(void)
{
	host_t *h, *next;

	/* free all existing entries */
	for(h = hosts; h != NULL; h = next) {
		next = h->next;
		free(h->ip);
		free(h->name);
		free(h);
	}
	hosts = NULL;
}

static void uninit(void)
{
	free_all();
	in_chain = 0;
}

int P_net_resolv_hosts_reread(void)
{
	FILE *f;
	char line[1024], *s, *ip, *name, *nname;

	if (!in_chain) {
		P_net_uninit_reg(uninit);
		in_chain = 1;
	}

	f = fopen(hosts_fn, "r");
	if (f == NULL)
		return -1;

	free_all();

	/* parse the file */
	while(!(feof(f))) {
		*line = '\0';
		fgets(line, sizeof(line), f);
		s = line;
		while((*s == ' ') || (*s == '\t')) s++;
		switch(*s) {
			case '#':
			case '\0':
			case '\r':
				break;
			default:
				ip = name = s;
				while (name != NULL) {
					nname = strpbrk(name, " \t\r\n");
					if (nname != NULL) {
						*nname = '\0';
						nname++;
						while((*nname == ' ') || (*nname == '\t') || (*nname == '\r') || (*nname == '\n')) nname++;
						if (*nname == '\0')
							nname = NULL;
					}
					if (name != ip)
						put(ip, name);
					name = nname;
				}
		}
	}
	fclose(f);
	return 0;
}

/* look at mtime fo the hosts file and reread file if it's different
   from what we've seen last */
static void check_reread()
{
	P_time_t mt;
	mt = P_net_mtime(hosts_fn);
	if (mt != hosts_seen) {
		hosts_seen = mt;
		P_net_resolv_hosts_reread();
	}
}

const char *P_net_resolv_hosts_i2n(const char *ip)
{
	host_t *h;

	check_reread();
	for(h = hosts; h != NULL; h = h->next)
		if (strcmp(h->ip, ip) == 0)
			return h->name;

	return NULL;
}

const char *P_net_resolv_hosts_n2i(const char *name)
{
	host_t *h;

	check_reread();
	for(h = hosts; h != NULL; h = h->next)
		if (strcmp(h->name, name) == 0)
			return h->ip;

	return NULL;
}
